@@ -36,11 +36,11 @@ def record_to_model(record: Record, model_class: ModelType) -> Model:
     Returns:
         Model: Model instance.
     """
-    new_record = model_class()
+    mob = model_class()
     for k,v in record.items():
-        new_record.Meta._fromdb_.append(k)
-        setattr(new_record, k, v)
-    return new_record
+        mob.Meta._fromdb_.append(k)
+        setattr(mob, k, v)
+    return mob
 
 
 class Pool(object):
@@ -155,10 +155,11 @@ class DB(object):
         con (Connection): Connection. Defaults to None.
     """
 
-    def __init__(self, pool: Pool, con: Connection=None):
+    def __init__(self, pool: Pool, con: Connection=None, sudo=False):
         self._pool = pool
         self._con = con
         self.DATA_NO_CHANGE = 'DATA_NO_CHANGE_TRIGGERED'
+        self.sudo = sudo
 
     def corp(self) -> Union[asyncpg.pool.Pool, Connection]:
         """Return the connection if available, otherwise return a Pool.
@@ -292,7 +293,7 @@ class DB(object):
             (str, list): query, args
         """
         data = mob.Meta._fields_
-        new_data_gen = mob.__class__._get_FieldValue_data_valid_(data, up=True, validate_all=True, mob=mob)
+        new_data_gen = mob.__class__._get_FieldValue_data_valid_(data, up=True, validate_all=True, mob=mob, sudo=self.sudo)
         columns = []
         values = []
         markers = []
@@ -329,7 +330,7 @@ class DB(object):
         """
         pkval = getattr(mob, mob.__class__._get_pk_()) #save method depends on it's AttributeError
         data = mob.Meta._fields_
-        new_data_gen = mob.__class__._get_FieldValue_data_valid_(data, up=True, mob=mob)
+        new_data_gen = mob.__class__._get_FieldValue_data_valid_(data, up=True, mob=mob, sudo=self.sudo)
         colval = []
         values = []
         c = 0
@@ -390,6 +391,7 @@ class DB(object):
             (Any): Value of primary key of the inserted row
         """
         query, args = self.get_insert_query(mob, reset=True)
+        print(query, args)
         await mob._pre_insert_(self)
         pkval = await self.fetchval(query, *args, timeout=timeout)
         if pkval is not None:
@@ -858,7 +860,7 @@ class ModelQuery():
         """
         return self.q('WHERE')
 
-    def qfilter(self, no_ordering=False) -> 'ModelQuery':
+    def qfilter(self, no_ordering=False, select_cols=()) -> 'ModelQuery':
         """Initiate a filter.
 
         This initiates a `SELECT` query upto `WHERE`. You can then use the
@@ -874,12 +876,16 @@ class ModelQuery():
 
         Args:
             no_ordering (bool): Whether to remove the default ordering SQL. Defaults to False.
+            select_cols (list): List of columns to select. Defaults to empty list which means all columns.
 
         Returns:
             ModelQuery: returns self to enable method chaining
         """
         if not self.__filter_initiated:
-            down_fields = ','.join([Q(x) for x in self.model._get_fields_(up=False)]) #type: ignore
+            valid_cols = self.model._get_fields_(up=False)
+            if select_cols:
+                valid_cols = set(valid_cols) & set(select_cols)
+            down_fields = ','.join([Q(x) for x in valid_cols]) #type: ignore
             self.reset().q(f'SELECT {down_fields} FROM "{self.model._get_db_table_()}" WHERE') #type: ignore
             self.__filter_initiated = True
             order_by = self.ordering
@@ -924,7 +930,7 @@ class ModelQuery():
         query = f'{self.start_query_str} {query} {self.end_query_str}'
         return query, self._args
 
-    async def fetch(self, timeout: float|None = None) -> Union[List[ModelBase], List[Record]]:
+    async def fetch(self, timeout: float|None = None, model_class='self') -> Union[List[ModelBase], List[Record]]:
         """Run query method `fetch` that returns the results in model class objects
 
         Returns the results in model class objects.
@@ -935,10 +941,11 @@ class ModelQuery():
         Returns:
             List[Model]: List of model instances.
         """
+        model_class = self.model if model_class == 'self' else model_class
         query, args = self.getq()
-        return await self.db.fetch(query, *args, timeout=timeout, model_class=self.model)
+        return await self.db.fetch(query, *args, timeout=timeout, model_class=model_class)
 
-    async def fetchrow(self, timeout: float|None = None) -> Union[ModelBase, Record]:
+    async def fetchrow(self, timeout: float|None = None, model_class='self') -> Union[ModelBase, Record]:
         """Make a query and get the first row.
 
         Resultant record is mapped to model_class object.
@@ -949,8 +956,9 @@ class ModelQuery():
         Returns:
             model_clas object or None if no rows were selected.
         """
+        model_class = self.model if model_class == 'self' else model_class
         query, args = self.getq()
-        return await self.db.fetchrow(query, *args, timeout=timeout, model_class=self.model)
+        return await self.db.fetchrow(query, *args, timeout=timeout, model_class=model_class)
 
     async def fetchval(self, column: int = 0, timeout: float|None = None) -> Any:
         """Run the query and return a column value in the first row.
@@ -977,7 +985,7 @@ class ModelQuery():
         query, args = self.getq()
         return await self.db.execute(query, *args, timeout=timeout)
 
-    async def get(self, *vals: Any, col: str = '', comp: str = '=$1') -> Union[ModelBase, Record]:
+    async def get(self, *vals: Any, col: str = '', comp: str = '=$1', many=False, select_cols=()) -> Union[ModelBase, Record] | List[Union[ModelBase, Record]]:
         """Get the first row found by column and value.
 
         If `col` is not given, it defaults to the primary key (`pk`) of
@@ -1003,13 +1011,17 @@ class ModelQuery():
             *vals (any): Values to compare. Must be referenced with $1, $2 etc.. in `comp`.
             col (str, optional): Column name. Defaults to the primary key.
             comp (str, optional): Comparison. Defaults to '=$1'.
+            many (bool, optional): If True, return all rows. Defaults to False.
+            select_cols (list, optional): List of columns to select. Defaults to empty list which means all columns.
 
         Returns:
             model_clas object or None if no rows were selected.
         """
         if not col:
             col = self.model.Meta.pk    #type: ignore
-        return await self.reset().qfilter().qc(col, comp, *vals).fetchrow()
+        if not many:
+            return await self.reset().qfilter(select_cols=select_cols).qc(col, comp, *vals).fetchrow()
+        return await self.reset().qfilter(select_cols=select_cols).qc(col, comp, *vals).fetch()
 
 SERIALIZABLE = 'serializable'
 REPEATABLE_READ = 'repeatable_read'
