@@ -7,7 +7,11 @@ __license__ = '[BSD](http://www.opensource.org/licenses/bsd-license.php)'
 __version__ = '0.0.1'
 
 import copy, inspect
+from enum import Enum
 from typing import Any, Optional, Callable, Tuple, Dict, List, Union
+from decimal import Decimal
+from typing_extensions import Annotated
+from pydantic import Field as pdField, AfterValidator, ValidationError
 from morm.void import Void
 import morm.exceptions as ex
 
@@ -34,6 +38,17 @@ def nomodify(value: Any) -> Any:
     """
     return value
 
+def wrap_validator(func, help=''):
+    def wrapped(v):
+        if func(v):
+            return v
+        msg = f"Validation failed for {v}"
+        if help == 'auto':
+            msg += ". Reason: " + inspect.getsource(func).split('\n')[0].strip()
+        elif help:
+            msg += f". Reason: {help}"
+        raise ValidationError(msg)
+    return wrapped
 
 class ColumnConfig():
     """Config container for each column/feild.
@@ -134,7 +149,7 @@ class ColumnConfig():
         queries = []
         msgs = []
         if self.conf['sql_type'] != prev.conf['sql_type']:
-            q = f'ALTER TABLE "{table_name}" ALTER COLUMN "{self.conf["column_name"]}" SET DATA TYPE {self.conf["sql_type"]};'
+            q = f'ALTER TABLE "{table_name}" ALTER COLUMN "{self.conf["column_name"]}" SET DATA TYPE {self.conf["sql_type"]} USING "{self.conf["column_name"]}"::{self.conf["sql_type"]};'
             queries.append(q)
             msg = f"\n* > MODIFY: {prev.conf['column_name']}: {prev.conf['sql_type']} --> {self.conf['sql_type']}"
             msgs.append(msg)
@@ -223,6 +238,34 @@ def sql_val(value: Any, sql_type: str) -> str|int|float|bool|None:
     except:
         return f"'{value}'"
 
+def sqlTypeToNative(sql_type: str, optional=False, containerType=None) -> Any:
+    '''Convert sql type to python native type.
+
+    Returns:
+        Tuple[Any, Any]: (native_type, raw_type)
+    '''
+    dtype = str
+    if 'interval' in sql_type:
+        dtype = str
+    if 'int' in sql_type or 'serial' in sql_type:
+        dtype = int
+    elif 'float' in sql_type or 'double' in sql_type or 'real' in sql_type:
+        dtype = float
+    elif 'money' in sql_type or 'numeric' in sql_type or 'decimal' in sql_type:
+        dtype = Decimal
+    elif 'bool' in sql_type:
+        dtype = bool
+    elif 'json' in sql_type:
+        dtype = Union[Dict[str, Any], List]
+    elif 'bytea' in sql_type:
+        dtype = bytearray
+
+    if '[' in sql_type or 'array' in sql_type.lower():
+        containerType = List
+    if optional:
+        return Optional[dtype] if containerType is None else Optional[containerType[dtype]], dtype
+    return dtype if containerType is None else containerType[dtype], dtype
+
 
 class Field(object):
     """Initialize the Field object with data type (sql).
@@ -253,19 +296,32 @@ class Field(object):
 
     Args:
         sql_type (str): Data type in SQL.
+        max_length (int, optional): Maximum length of the field. Defaults to None.
+        max_digits (int, optional): Maximum digits for numeric type. Defaults to None.
+        decimal_places (int, optional): Decimal places for numeric type. Defaults to None.
+        array_dimension (Tuple[int, ...], optional): Array dimension. Defaults to ().
         sql_onadd (str): sql to add in ADD clause after 'ADD COLUMN column_name data_type'
         sql_ondrop (str): Either 'RESTRICT' or 'CASCADE'.
         sql_alter (Tuple[str]): Alter column queries; Example: ('ALTER TABLE "{table}" ALTER COLUMN "{column}" DROP DEFAULT',).
         sql_engine (str): db engine, postgresql, mysql etc.. Defaults to 'postgresql'
         default (Any, optional): Pythonic default value (can be a callable). Defaults to Void. (Do not use mutable values, use function instead)
         value (Any, optional): Set a value that will prevail unless changed manually. Can be a function. Useful to make updated_at like fields.
+        unique (bool, optional): Whether the field is unique. Defaults to False.
+        index (str, optional): Index type. Defaults to None. 'btree', 'hash', 'gin', 'gist', prepend with - to remove the index.
         choices (Tuple[Tuple[str, Any], ...], optional): choices tuple: (('Name of choice', 'value'), ...)
         help_text (str, optional):  help text to describe this field.
         validator (callable, optional): A callable that accepts exactly one argument. Validates the value in `clean` method. Defaults to always_valid.
         modifier (callable, optional): A callable that accepts exactly one argument. Modifies the value if validation fails when the `clean` method is called.. Defaults to nomodify.
         fallback (bool, optional): Whether invalid value should fallback to default value suppressing exception. (May hide bugs in your program)
+        sudo (bool, optional): Mark this field as requiring elevated permission (for something that you will implement in your app). Defaults to None.
+        groups (tuple, optional): Groups that this field belongs to. Defaults to ().
+        allow_null (bool, optional): Allow null value. Defaults to False.
     """
     def __init__(self, sql_type: str,
+                max_length: Optional[int]=None, # added to sql_type e.g varchar(max_length)
+                max_digits: Optional[int]=None, # added to sql_type e.g numeric(max_digits, decimal_places)
+                decimal_places: Optional[int]=None, # added to sql_type e.g numeric(max_digits, decimal_places)
+                array_dimension: Tuple[int, ...] = (),
                 sql_onadd='',
                 sql_ondrop='',
                 sql_alter: Tuple[str, ...] = (),
@@ -273,16 +329,48 @@ class Field(object):
                 default: Any=Void,
                 value: Any=Void,
                 unique=False,
-                index=None, # 'btree', 'hash', 'gin', 'gist', prepend with - to remove the index.
+                index: Tuple[str] | str | None=None, # 'btree', 'hash', 'gin', 'gist', prepend with - to remove the index. Can be a tuple or comma separated string to specify multiple indexes.
                 choices: Tuple[Tuple[str, Any], ...] = (),
                 help_text: str = '',
                 validator: Callable=always_valid,
+                validator_text: str = '',
                 modifier: Callable=nomodify,
-                fallback=False,): # if you add new param here, update __repr__ method
+                fallback=False,
+                sudo=None,
+                groups: Tuple[str, ...]=(),
+                allow_null=None,
+                drop_constraints: Tuple[str, ...] = (),
+            ): # if you add new param here, update __repr__ method
         # Rules for using a variable name here as local variables go into the self._json_:
         # 1. Must precede with underscore if not in the parameter list
         # 2. Make sure to exclude unnecessary variables in the self._json_ like 'self' etc..
         _init_args = list(locals().keys())[1:] # this must be the first line here in __init__
+        self.sudo = sudo
+        self.groups = groups
+        self.max_length = max_length
+        self.max_digits = max_digits
+        self.decimal_places = decimal_places
+        self.validator_text = validator_text
+        if isinstance(default, Enum):
+            default = default.value
+        if isinstance(value, Enum):
+            value = value.value
+        self.native_type, self.native_type_raw = sqlTypeToNative(sql_type, optional=default is not Void, containerType=None if not array_dimension else List)
+        if max_length and self.native_type_raw is not str:
+            raise ValueError(f"max_length is only valid for types that are string like, {sql_type} given.")
+        if max_length or max_digits:
+            if '(' in sql_type:
+                raise ValueError(f"Please remove (max_length) or (max_digits) from sql_type: {sql_type} as you are using max_length or max_digits explicitly.")
+        if max_length:
+            sql_type = f'{sql_type}({max_length})'
+        elif max_digits and decimal_places:
+            sql_type = f'{sql_type}({max_digits}, {decimal_places})'
+        elif max_digits:
+            sql_type = f'{sql_type}({max_digits})'
+        if array_dimension:
+            if '[' in sql_type or 'array' in sql_type.lower():
+                raise ValueError(f"Please remove array indicator from sql_type: {sql_type} as you are using array_dimension.")
+            sql_type = f'{sql_type}'+ ''.join([f'[{x}]' for x in array_dimension])
 
         self.sql_type = sql_type
         _unique_constraint = '__UNQ_{table}_{column}__'
@@ -290,23 +378,40 @@ class Field(object):
             _sql_unique = 'ALTER TABLE "{table}" ADD CONSTRAINT "%s" UNIQUE ("{column}");' % (_unique_constraint,)
         else:
             _sql_unique = 'ALTER TABLE "{table}" DROP CONSTRAINT IF EXISTS "%s";' % (_unique_constraint,)
-        if index:
-            idx_remove = True if index[0] == '-' else False
-            if idx_remove:
-                index = index[1:]
-            if index not in ['btree', 'hash', 'gin', 'gist', 'spgist', 'brin']:
-                raise ValueError(f"Invalid index type: {index}")
-            _index_name = '__IDX_{table}_{column}_'+index+'__'
-            if not idx_remove:
-                _sql_index = 'CREATE INDEX IF NOT EXISTS "%s" ON "{table}" USING %s ("{column}")' % (_index_name, index)
-            else:
-                _sql_index = 'DROP INDEX IF EXISTS "%s"' % (_index_name,)
         sql_alter = (_sql_unique, *sql_alter)
+        _sql_index = ''
         if index:
+            if isinstance(index, str):
+                index = index.split(',')
+            for idx in index:
+                _idx_remove = True if idx[0] == '-' else False
+                _idx_ops = ''
+                if ':' in idx:
+                    idx, _idx_ops = idx.split(':', 1)
+                if _idx_remove:
+                    idx = idx[1:]
+                if idx.lower() not in ['btree', 'hash', 'gin', 'gist', 'spgist', 'brin']:
+                    raise ValueError(f"Invalid index type: {idx}")
+                _index_name = '__IDX_{table}_{column}_'+idx+'__'
+                if not _idx_remove:
+                    _sql_index += 'CREATE INDEX IF NOT EXISTS "%s" ON "{table}" USING %s ("{column}" %s);' % (_index_name, idx, _idx_ops)
+                else:
+                    _sql_index += 'DROP INDEX IF EXISTS "%s";' % (_index_name,)
+        if _sql_index:
             sql_alter = (*sql_alter, _sql_index)
+        if allow_null:
+            sql_alter = (*sql_alter, 'ALTER TABLE "{table}" ALTER COLUMN "{column}" DROP NOT NULL;')
+        elif allow_null is False:
+            sql_alter = (*sql_alter, 'ALTER TABLE "{table}" ALTER COLUMN "{column}" SET NOT NULL;')
         # handle default
         if isinstance(default, (int, float, str, bool)) or is_sql_array(default):
-            sql_alter = (*sql_alter, "ALTER TABLE \"{table}\" ALTER COLUMN \"{column}\" SET DEFAULT "+f"{sql_val(default, sql_type)};")
+            sql_alter = (*sql_alter, "ALTER TABLE \"{table}\" ALTER COLUMN \"{column}\" SET DEFAULT "+f"{sql_val(default, sql_type)}::{sql_type};")
+
+        for drc in drop_constraints:
+            if drc == 'fk':
+                drc = '__FK_{table}_{column}__'
+            sql_alter = (*sql_alter, 'ALTER TABLE "{table}" DROP CONSTRAINT IF EXISTS "%s";' % (drc,))
+
         self.sql_conf = ColumnConfig(sql_type=sql_type, sql_onadd=sql_onadd, sql_ondrop=sql_ondrop, sql_alter=sql_alter, sql_engine=sql_engine)
         self.validator = validator
         self.modifier = modifier
@@ -330,13 +435,14 @@ class Field(object):
         }
         args = locals()
         for k,v in args.items():
-            if k in ['self', '_init_args']: continue
+            if k in ['self', '_init_args'] or k.startswith('_'): continue
             if callable(v):
-                self._json_[k] = inspect.getsource(v)
+                v_src = inspect.getsource(v)
+                v_src = v_src.split('\n')[0].strip()+' ...'
+                self._json_[k] = v_src
             else:
                 self._json_[k] = 'Void' if v == Void else v
 
-
     def __eq__(self, other: 'Field') -> bool: # type: ignore
         return bool(other and self.sql_conf == other.sql_conf)
 
@@ -349,13 +455,19 @@ class Field(object):
             else:
                 try:
                     if callable(self.__dict__[k]):
-                        reprs.append(f'{k}={inspect.getsource(self.__dict__[k])}')
+                        k_src = inspect.getsource(self.__dict__[k])
+                        k_src = k_src.split('\n')[0].strip()+' ...'
+                        reprs.append(f'{k}={k_src}')
                     else:
                         reprs.append(f'{k}={repr(self.__dict__[k])}')
                 except KeyError: pass
         body = ', '.join(reprs)
         return f'{self.__class__.__name__}({body})'
 
+
+    def check_sudo(self, sudo: bool) -> bool:
+        return False if self.sudo and not sudo else True
+
     def to_json(self):
         res = self._json_.copy()
         res['_name'] = self.name
@@ -363,6 +475,41 @@ class Field(object):
         del res['_init_args']
         return res
 
+    def to_pydantic_override(self, dType) -> Tuple[Any, Dict[str, Any]]:
+        '''Override this method to return the type and a dict of pydantic field overrides.
+
+        you can return a different dType to change the type of the field in pydantic model.
+
+        Parameters:
+            dType: The type that is returned by the default to_pydantic method.
+
+        '''
+        return dType, {}
+
+    def to_pydantic(self, include_validator=True) -> Tuple[Any, pdField]:
+        """Get the pydantic field with type
+
+        Returns:
+            (dataType, pydantic.Field)
+        """
+        opts = {
+            'description': self.help_text,
+            'title': self.name.replace('_', ' ').title(),
+        }
+        if self.default is not Void:
+            k = 'default_factory' if callable(self.default) else 'default'
+            opts[k] = self.default
+        if self.max_length: opts['max_length'] = self.max_length
+        if self.max_digits: opts['max_digits'] = self.max_digits
+        if self.decimal_places: opts['decimal_places'] = self.decimal_places
+
+        # Final processing: overrides
+        dType, _opts = self.to_pydantic_override(self.native_type)
+        if include_validator:
+            dType = Annotated[dType, AfterValidator(wrap_validator(self.validator, help=self.validator_text))]
+        opts.update(_opts)
+        return dType, pdField(**opts)
+
     def __invert__(self):
         return self.name
 
@@ -377,6 +524,23 @@ class Field(object):
         self._name = v
         self.sql_conf.conf['column_name'] = v
 
+    def field_validator_error(self, value: Any, e=None) -> str:
+        """Get the error message for the validator
+
+        Args:
+            value (Any): value
+            e (Exception, optional): Exception. Defaults to None.
+
+        Returns:
+            str: error message
+        """
+        emsg = f"Value of type '{type(value).__name__}' did not pass validation check for field '{self.name}'"
+        if self.validator_text:
+            emsg += f". {self.validator_text}"
+        if e:
+            emsg += f". Reason: {str(e)}"
+        return emsg
+
     def clean(self, value: Any, fallback: bool=False):
         """Clean the value by calling validator -> modifier -> validator
 
@@ -395,16 +559,21 @@ class Field(object):
         Returns:
             Any: value
         """
-        if not self.validator(value):
-            value = self.modifier(value)
-        else:
+        try:
+            if not self.validator(value):
+                value = self.modifier(value)
+            else:
+                return value
+            # the value may have been changed, try to validate again
+            if not self.validator(value):
+                if fallback:
+                    return self.get_default()
+                raise ValueError(self.field_validator_error(value))
             return value
-        # the value may have been changed, try to validate again
-        if not self.validator(value):
-            if fallback:
-                return self.get_default()
-            raise ValueError("Value (%s) (type: %s) did not pass validation check for '%s'" % (str(value), type(value), self.name,))
-        return value
+        except ValidationError:
+            raise
+        except Exception as e:
+            raise ValueError(self.field_validator_error(value,e=e)) from e
 
     def get_default(self):
         """Get the default value.
