@@ -4,11 +4,12 @@
 __author__ = 'Md Jahidul Hamid <jahidulhamid@yahoo.com>'
 __copyright__ = 'Copyright © Md Jahidul Hamid <https://github.com/neurobin/>'
 __license__ = '[BSD](http://www.opensource.org/licenses/bsd-license.php)'
-__version__ = '0.0.2'
+__version__ = '0.0.3'
 
-import inspect
+import inspect, re
 from typing import Dict, List, Tuple, Any, Iterator, ClassVar
 import copy
+import pydantic
 from morm.fields.field import Field, FieldValue
 from morm.void import Void
 import morm.meta as mt      # for internal use
@@ -69,6 +70,8 @@ class ModelType(type):
                     meta_attrs[k] = v
 
         _set_meta_attr('proxy', False)
+        _set_meta_attr('sudo', None)
+        _set_meta_attr('groups', ())
         _set_meta_attr('pk', 'id')
         _set_meta_attr('ordering', ())
         _set_meta_attr('fields_up', ())
@@ -78,7 +81,9 @@ class ModelType(type):
         _set_meta_attr('exclude_values_up', {'':()}, mutable=True)
         _set_meta_attr('exclude_values_down', {'':()}, mutable=True)
         _set_meta_attr('ignore_init_exclude_error', True)
+        _set_meta_attr('unique_groups', {}, mutable=True)
         _set_meta_attr('_field_defs_', {}, internal=True, mutable=True)
+        _set_meta_attr('_field_groups_', {}, internal=True, mutable=True)
 
         if meta_attrs['proxy']:
             #proxy model inherits everything
@@ -105,7 +110,13 @@ class ModelType(type):
                 if meta_attrs['proxy'] and n in attrs:
                     raise ValueError(f"Proxy model '{class_name}' can not define new field: {n}")
                 v.name = n
+                if meta_attrs['sudo'] is not None and v.sudo is None:
+                    v.sudo = meta_attrs['sudo']
                 # v.sql_conf.conf['table_name'] = meta_attrs['db_table'] # Field must not contain table_name, because it is void when model is abstract and it gets inherited.
+                for g in v.groups:
+                    gs = meta_attrs['_field_groups_'].setdefault(g, [])
+                    if n not in gs:
+                        gs.append(n)
                 meta_attrs['_field_defs_'][n] = v
             elif n in attrs:
                 new_attrs[n] = attrs[n]
@@ -126,7 +137,10 @@ class ModelType(type):
         return super().__new__(mcs, class_name, bases, new_attrs)
 
     def __repr__(cls):
-        return inspect.getsource(cls)
+        src = inspect.getsource(cls)
+        # remove the function definitions as they may contain sensitive data
+        src = re.sub(r'\n\s+def\s+[\s\S]*$', '\n', src, flags=re.MULTILINE)
+        return src
 
     def __setattr__(self, k, v):
         raise NotImplementedError("You can not set model attributes outside model definition.")
@@ -220,6 +234,67 @@ class ModelType(type):
         else:
             raise AttributeError(f"No such field `{n}` in model `{self.__name__}`")
 
+    def _check_sudo_(self, sudo: bool, n: str='') -> bool|List[str]:
+        """Check if the given sudo can get access to the field
+        Or return a list of fields that can be accessed with the given
+        value of sudo.
+        """
+        if n:
+            return self.Meta._field_defs_[self._check_field_name_(n)].check_sudo(sudo)
+        return [k for k, v in self.Meta._field_defs_.items() if v.check_sudo(sudo)]
+
+    def _sudo_fields_(self) -> Iterator[str]:
+        """Yield field names that require elevated access
+
+        Yields:
+            str: field name
+        """
+        for k, v in self.Meta._field_defs_.items():
+            if v.sudo:
+                yield k
+
+    def _groups_(self) -> Tuple[str]:
+        """Get groups that this model belongs to
+
+        Returns:
+            Tuple[str]: Tuple of group names
+        """
+        return tuple(self.Meta.groups)
+
+    def _group_fields_(self, g: str) -> Tuple[str]:
+        """Get field names of a group
+
+        Args:
+            g (str): group name
+
+        Returns:
+            Tuple[str]: Tuple of field names
+        """
+        return tuple(self.Meta._field_groups_.get(g, []))
+
+    def _field_groups_(self, n) -> Tuple[str]:
+        """Get groups of a field
+
+        Args:
+            n (str): field name
+
+        Returns:
+            Tuple[str]: List of group names
+        """
+        return tuple(self.Meta._field_defs_[self._check_field_name_(n)].groups)
+
+    def _check_group_(self, g: str, n: str) -> bool:
+        """Check if a field belongs to a group
+
+        Args:
+            g (str): group name
+            n (str): field name
+
+        Returns:
+            bool: True if field belongs to the group
+        """
+        return n in self._group_fields_(g)
+
     def _get_fields_(self, up=False) -> Iterator[str]:
         """Yields field names that pass include/exclude criteria
 
@@ -257,16 +332,45 @@ class ModelType(type):
             res[k] = all_fields[k].to_json()
         return res
 
+    def _pydantic_(self, up=False, suffix=None, include_validators=None):
+        '''Create a pydantic model from the morm model
+
+        ### Parameters:
+
+        up: bool
+            Whether it's for up (data update) or down (data retrieval)
+        suffix: str
+            Suffix to append to the model name, default: _UP or _DOWN
+        include_validators: bool
+            Include validators for each fields, default: None. When None
+            It will include validators when up=True.
+
+        ### Returns:
+            pydantic model
+        '''
+        include_validators = up if include_validators is None else include_validators
+        name = self.__name__
+        u = '_UP' if up else '_DOWN'
+        if suffix: u = suffix
+        name = name + u
+        res = {}
+        fields = self._get_fields_(up)
+        all_fields = self._get_all_fields_()
+        for k in fields:
+            res[k] = all_fields[k].to_pydantic(include_validator=include_validators)
+        return pydantic.create_model(name, **res)
+
     def _run_validations_(self, k: str, v: FieldValue, mob=None) -> FieldValue:
         try:
             validator = getattr(mob, '_clean_' + k)
-            v.value = validator(v.value)
+            v._value = validator(v.value) # do not trigger the field validator
+            # and do not increase the change count
         except AttributeError:
-            v.value = v.value # at least trigger the value validation
+            pass
         return v
 
 
-    def _get_FieldValue_data_valid_(self, data: dict, up=False, validate_all=False, mob=None) -> Iterator[Tuple[str, Any]]:
+    def _get_FieldValue_data_valid_(self, data: dict, up=False, validate_all=False, mob=None, sudo=False) -> Iterator[Tuple[str, Any]]:
         """Yields valid key,value pairs from data.
 
         Validity is checked against include/exclude key/value criteria.
@@ -293,8 +397,11 @@ class ModelType(type):
                 continue
             if not self._is_valid_value_(k, v.value, exclude_values):
                 continue
-            if not validate_all: # run validations for to-be-changed fields only
+            if not validate_all and v.value_change_count > 0: # run validations for to-be-changed fields only
                 v = self._run_validations_(k, v, mob)
+            # check sudo
+            if not v._field.check_sudo(sudo):
+                raise ValueError(f"Field {k} requires elevated access.")
             yield k, v
 
 
@@ -379,6 +486,8 @@ class ModelBase(metaclass=ModelType):
         db_table = Void
         abstract = True
         proxy = False
+        sudo = None
+        groups = ()
         ordering = ()
         fields_up = ()
         fields_down = ()
@@ -387,9 +496,11 @@ class ModelBase(metaclass=ModelType):
         exclude_values_up = {'':()}
         exclude_values_down = {'':()}
         ignore_init_exclude_error = True
+        unique_groups = {}
 
         #internal
         _field_defs_: Dict[str, Field]
+        _field_groups_: Dict[str, List[str]]
         _fields_: Dict[str, FieldValue]
         _fromdb_: List[str]
         _initializing_: bool = False
@@ -432,6 +543,9 @@ class ModelBase(metaclass=ModelType):
         else:
             super().__delattr__(k)
 
+    def __getitem__(self, k):
+        return getattr(self, k)
+
     def __getattr__(self, k):
         Meta = self.__dict__['Meta']
         fields = Meta._fields_
@@ -442,6 +556,9 @@ class ModelBase(metaclass=ModelType):
             raise AttributeError(f'Invalid attempt to access field `{k}`. It is excluded using either exclude_fields_down or exclude_values_down in {self.__class__.__name__} Meta class. Or it does not have any valid value.')
         raise AttributeError
 
+    def __setitem__(self, k, v):
+        setattr(self, k, v)
+
     def __setattr__(self, k: str, v):
         if k == 'Meta':
             raise AttributeError(f"Name '{k} is reserved. You should not try to change it.")
@@ -458,8 +575,11 @@ class ModelBase(metaclass=ModelType):
         if k in self.Meta._fromdb_:
             self.Meta._fromdb_.remove(k)
             if self.__class__._is_valid_down_(k, v):
+                # value coming from DB should not increment change counter
+                fields[k]._ignore_first_change_count_ = True
                 fields[k].value = v
             elif self.__class__._is_valid_up_(k, v):
+                # also suppress counting when setting initial up-value from DB
                 fields[k]._ignore_first_change_count_ = True
                 fields[k].value = v
             elif self.__class__.Meta.ignore_init_exclude_error and self.Meta._initializing_: # ignore this error at init
