@@ -3,7 +3,7 @@ import unittest
 import os, sys
 import tempfile
 import shutil
-from morm import datetime
+from morm import dt
 
 
 LOGGER_NAME = 'morm-test-admin-'
@@ -13,8 +13,8 @@ log = logging.getLogger(LOGGER_NAME)
 
 class TestMethods(unittest.TestCase):
     def test_main(self):
-        print(datetime.timestamp())
-        print(datetime.timestampu())
+        print(dt.timestamp())
+        print(dt.timestampu())
 
 
 
